pro writefile,wave,sp,err,bkgd,bkgderr,fname,exp=exp

; write a data file in the format comprehensive to
;  the routine 'read_data'

 get_lun,unit
 close,unit
 openw,unit,fname
 if n_elements(exp) gt 0 then printf,unit,'#',exp

 ndim=n_elements(wave)
 ndim1=n_elements(sp)
 ndim2=n_elements(err)
 ndim3=n_elements(bkgd)
 if ndim ne ndim1 or ndim2 ne ndim1 or ndim ne ndim2 or ndim ne ndim3 then begin
	print,'data have not same dimension in writefile',ndim,ndim1,ndim2,ndim3
	stop
 endif
 FOR i=0,ndim-1 DO BEGIN
    printf,unit,format='(5F11.4,x)',wave(i),sp(i),err(i),bkgd(i),bkgderr(i)
 ENDFOR
 close,unit
 free_lun,unit

return
end

;-----------------------------------------------

pro count_cols,datafile,ncols

get_lun,unit

openr, unit, datafile
cc = '#'
while strmid(cc,0,1) eq '#' DO readf,unit,cc
test=strtrim(cc,1)

test=strcompress(test)
b0=strmid(test,0,1)
rlen=strlen(test)
ncols=1
for i=1,rlen-1 DO begin 
 b1=strmid(test,i,1) 
 if b1 eq ' ' and b1 ne b0 then ncols=ncols+1 
 b0=b1 
endfor

close,unit
free_lun,unit

return
end

;-----------------------------------------------

pro read_data_ascii,unit,datafile

COMMON DATA,wave,sp,err,bkgd,good,yc,exp

print,'reading from datafile: ',datafile
result = READ_ASCII( datafile, COMMENT_SYMBOL='#' , COUNT=tcount , DELIMITER=' ', HEADER=head , NUM_RECORDS=rcount )

print,tcount,' entries'

x=result(0,*).field1
wave=x(0,*)
sp=x(1,*)
err=sqrt(sp)
bkdg=fltarr(tcount)

;test=min(size(x,/dimensions))
test=min(size(x))

if test(0) eq 3 then bkgd=x(2,*) else if test(0) eq 5 then begin
  err=x(2,*)
  bkdg=x(3,*)
endif

return
end

;-----------------------------------------------

pro read_data,unit,datafile

; read spectral data from formatted data file
; into global variables

COMMON DATA,wave,sp,err,bkgd,good,yc,exp

count_cols,datafile,ncols

print,'reading from datafile: ',datafile
close,unit
openr, unit, datafile
cc = ' '
j = long(0)
nj = 0
while(not eof(unit)) do begin
  readf,unit,cc
  if strmid(cc,0,1) ne '#' then j = long(j) + long(1) else nj=long(nj)+long(1)
endwhile
number = j
print,number,' entries'
print,nj,' comments'
print,ncols,' columns'
close,unit

a=fltarr(ncols,number)

openr, unit, datafile
for i=long(0),long(nj-1) DO readf,unit,cc
exp=1.
exp=double(strmid(cc,1,25))
readf,unit,a
close,unit

  wave=a(0,*)
  sp=a(1,*)
  err=sqrt(sp)
  bkgd = fltarr(number)
  if ncols eq 3 then bkgd=a(2,*) else if ncols eq 5 then begin
   err=a(2,*)
   bkgd=a(3,*) 
  endif

count=number
wave=wave(0:count-1)
sp=sp(0:count-1)
err=err(0:count-1)
bkgd=bkgd(0:count-1)

return
end

;-----------------------------------------------

;pro likelihood, x,ncount,backarr,yc,ampout,like
function likelihood,ampout
;   determine -2*log(likelihood) which is to be minimized

COMMON DATintv,x,dx,ncount,bbb,errcount,count
COMMON DATA,wave,sp,err,bkgd,good,yc,exp

 ndim=n_elements(yc(0,*))
 source=fltarr(ndim)
 FOR i=0,ndim-1 DO source(i)=total(ampout*yc(*,i))
 sb=bbb+source

sel=where(sb gt 0) ;prevent log to fail
 like = -2.*total((-sb(sel)) + float(ncount(sel))*alog(sb(sel)))

return,like
end

;-----------------------------------------------

function like_aux,P

@fit.inc

for i=0,nlines-2 DO if P(i) lt 0. or finite(P(i)) eq 0 then return,100000.*randomu(iter)

ampout(seliter)=P(0:max([nlines-2,0]))
ampout(nzline)=ampout(nzline)+(v-total(f*ampout))/f(nzline)

;ci=yc##ampout+bbb
like=likelihood(ampout)
;likelihood, x,ncount,bbb,yc,ampout,like
;nb=total(f*ampout)-v
;lagr=0.

;nzline=0
;while f(nzline) eq 0. DO nzline=nzline+1
;index=where(findgen(nlines) ne nzline)
;an=total(ncount*yc(nzline,*)/ci)
;lagr=(1.-an)/f(nzline)

;lagr=0.
;index=where(f ne 0.0)
;for i=0,nlines-2 DO begin
;    nzline=index(i)
;    an=total(ncount*yc(nzline,*)/ci)
;    lagr=lagr+(1.-an)/f(nzline)
;endfor

;index=where(f ne 0.0)
;ndim=n_elements(yc(0,*))
;gsumi=fltarr(ndim)
;for i=0,ndim-1 DO gsumi(i)=total(yc(index,i))
;lagr=(nlines-total(ncount*gsumi/ci))/total(f(index))

;like=like+lagr*nb

if finite(like) eq 1 then return,like else return,50000.
end

;-----------------------------------------------

Pro iteration, yc,ampout
;
;       returns fitted amplitude for given count number ncount(0..ndim-1)
;    with background bbb and given gauss function yc(0..nline:0..ndim-1)
;
COMMON iteration,iter,f,seliter,v,ampstart,nzline
COMMON DATintv,x,dx,ncount,bbb,errcount,count

nlines=n_elements(yc(*,0))
ndim=n_elements(yc(0,*))

;f=fltarr(nlines)
;f=[1]
;f=[1.,-1.,-1.]
;v=0.
;f=[0,0,1]

ampout=ampstart
ampnew=ampstart
npars=800
iter=0

IF total(abs(f)) eq 0 THEN BEGIN
;    iteration w/o auxiliary condition with fix point equation
  dfix=fltarr(nlines)
  ifix=intarr(nlines)
  nfix=0
  ci=yc##ampout+bbb
  for lline=0,nlines-1 DO BEGIN
    dfix(lline) = total(ncount*yc(lline,*)/ci^2*(ci-yc(lline,*)*ampout(lline)))
    if dfix(lline) lt 1. then begin
	ifix(nfix)=lline
	nfix=nfix+1
    endif
  endfor
if nfix gt 0 then ifix=ifix(0:nfix-1)
;print,'ifix=',ifix

 jump1: iter = iter + 1
  ci=yc##ampout+bbb
  FOR lline=0,nlines-1 DO BEGIN
      ampnew(lline)=ampout(lline)*total(ncount*reform(yc(lline,*))/ci)
      if (abs(ampnew(lline)) lt 0.001) then ampnew(lline) = 0.
  ENDFOR
;  iteration finished when correction terms become small
  if iter lt npars then begin
     if (max(abs((ampnew-ampout)/max([1e-4,ampout]))) ge 0.00001) then begin
        ampout = ampnew
        goto, jump1
     endif
  endif else begin
     print,'iteration exceeded',iter
     return
  endelse
  ampout = ampnew

ENDIF ELSE BEGIN
;    iteration with auxiliary condition total(f*ampout)==v
;     with numerical minimization

  nzline=0
  while f(nzline) eq 0. DO nzline=nzline+1
  seliter=where(findgen(nlines) ne nzline,nzcount)
  if nzcount eq 0 then seliter=fltarr(1)
  tol=1e-4
  if nlines gt 2 then begin
     xi=fltarr(nlines-1,nlines-1)
     FOR i=0,nlines-2 DO xi(i,i)=1.0
     P=ampstart(seliter)
     POWELL,P,xi,tol,llik,'like_aux',iter=iter
  endif else begin
     ax=0.
     bx=ampstart(seliter(0))/2.
     cx=ampstart(seliter(0))*2.
     fw=like_aux(ax)
     fx=like_aux(bx)
     fv=like_aux(cx)
     brent,npars,ax,bx,cx,tol,xmin,fw,fx,fv,idlam,iter,'like_aux'
  endelse

ENDELSE

return
end

;-----------------------------------------------

function gcurve,x,x0,sig,delx
xx = (exp(-0.5*((x-x0)/sig)^2))/(sig*(2.*!pi)^0.5)
return,delx*xx
end

;-----------------------------------------------

function ecurve,x,x0,sig,delx
 if sig eq 0. then sig=1e-2
 xx = (1. - (x-x0)^2/sig)
 sel=where(xx lt 0,c)
 if c gt 0. then xx(sel) = 0.
 xx = xx^0.5
 xx = xx*!pi*0.5/sig
 test=total(delx*xx)
return,delx*xx/test
end

;-----------------------------------------------

pro num_profile_init

COMMON DATintv,x,dx,ncount,bbb,errcount,count
COMMON fit_PARA,lamstart,lamend,nlines,prof,idlam,app_s,app_b,lsig,usig,sigunit,ilamit,isigit,sbgit,bgsmooth,dlam,beta
COMMON filenames,datafile,conffile,proffile
COMMON num_prof,template,prcount,dxt,x0i,xt,x0

 get_lun,unit
 openr,unit,proffile
 a='#'
 while strmid(a,0,1) eq '#' DO readf,unit,a
 prcount=0
 while eof(unit) eq 0 do begin & readf,unit,a & prcount=prcount+1 & endwhile
 close,unit

 openr,unit,proffile
 xt=fltarr(prcount)
 template=fltarr(prcount)

 a='#'
 while strmid(a,0,1) eq '#' DO readf,unit,a
 for i=0,prcount-1 DO begin
  readf,unit,a1,a2
  xt(i)=a1
  template(i)=a2
 endfor
 close,unit
 free_lun,unit

;   normalize
 tmax=total(template)
 sel=where(template gt 0)
  test=template(sel)
  template=fltarr(prcount)
 template(sel)=test(sel)/tmax
; find peak and peak position x0 in template profile
 tmax=max(template,x0i)
 x0=xt(x0i)
;determine length of investigation grid x.vec (in A)*/
 dx=x(count-1)-x(0)
;  determine length of profile template grid xt.vec (in A) 
 dxt=xt(prcount-1)-xt(0)
return
end

;   put the template profile at position(s) llambda in x grid

pro num_profile,llambda,lsigma,yc

COMMON DATintv,x,dx,ncount,bbb,errcount,count
COMMON fit_PARA,lamstart,lamend,nlines,prof,idlam,app_s,app_b,lsig,usig,sigunit,ilamit,isigit,sbgit,bgsmooth,dlam,beta
COMMON filenames,datafile,conffile,proffile
COMMON num_prof,template,prcount,dxt,x0i,xt,x0

yc=fltarr(nlines,count)
for j=0,nlines-1 do for i=0,count-2 Do begin
  k=x0i-(llambda(j)-x(i))/dxt*prcount
  if k gt 0 and k lt prcount then yc(j,i)=template(k)+(template(k+1)-template(k))* (x(i)-xt(k)+x0-llambda(j))/(x(i+1)-x(i)) else yc(j,i)=0.
 endfor

return
end

;-----------------------------------------------

pro profile,llambda,lsigma,yc

COMMON DATintv,x,dx,ncount,bbb,errcount,count
COMMON init,nshift
COMMON fit_PARA,lamstart,lamend,nlines,prof,idlam,app_s,app_b,lsig,usig,sigunit,ilamit,isigit,sbgit,bgsmooth,dlam,beta

;if n_elements(nshift) eq 0 then begin
; ndim=n_elements(x)
; i=2
; if ndim gt 500 then begin
;   while 1.*ndim/i ne fix(ndim/i) DO i=i+1
;   nshift=i
; endif else nshift=1
;endif
nshift=1
y=fltarr(count)

FOR lline=0,nlines-1 DO BEGIN
  if prof(lline) eq 0 then begin
;     Gauss
;   (gexp=exponent in gauss function)
   nint=1
   if nint eq 1 then begin
    gexp = -(x - llambda(lline))/lsigma(lline)
    y  = gaussint(gexp)
    ndim = n_elements(y)
    ys = shift(y,+nshift)
    ys = (ys-y)/nshift
;   ys = (mean([ys(i-2),ys(i-1),ys(i)])-mean([y(i-2),y(i-1),y(i)]))/nshift
;   ys = ys - y
    FOR i=nshift,ndim-1 DO yc(lline,i) = ys(i)
;   yc(lline,0)=0.
   endif else yc(lline,*)=gcurve(x,llambda(lline),lsigma(lline),dx)
  endif
  if prof(lline) eq 1 then begin
;    Lorentz
    myy  = lsigma(lline) / 2.0
   myy2 = myy * myy
   for i=0,nshift-1 DO begin
    y(count-1)=0
    y(i)=0.
   endfor
   for i=nshift,count-1 DO begin
    myx = x(i)-llambda(lline)
    y(i)=((myy/!pi) / ( (myx * myx) + myy2))^beta(lline)
    y(i) = y(i)*(x(i) - x(i-nshift))/(1.*nshift)
   endfor
   yc(lline,*)=y/total(y)
  endif
  if prof(lline) eq 2 then begin
;     elliptical
   yc(lline,*)=ecurve(x,llambda(lline),lsigma(lline),dx)
  endif
ENDFOR

return
end
;-----------------------------------------------

pro callik,llambda,lsigma,ampout,like,yc
;
;  determine profiles yc from lambdas and sigmas
;  determine then a good amplitude 'ampout' for yc and
;  a quality 'like' of the fit in terms of -2*log(likelihood)
;

COMMON DATintv,x,dx,ncount,bbb,errcount,count
COMMON num_prof,template,prcount,dxt,x0i

  if prcount eq 0 then profile,llambda,lsigma,yc else num_profile,llambda,lsigma,yc
  iteration, yc,ampout
  like=likelihood(ampout)

return
end

;-----------------------------------------------

pro penalty,P,lP,uP,flag,dist
;   returns number of entries 'flag' in P
;    with P<lP or P>uP
;   which is used as a criterium for a penalty

 flag=0
 N=n_elements(P)
 if N ne n_elements(lP) OR N ne n_elements(uP) then $
    message,'Penalty can not run with different dimensions'
 index=where(P gt uP or P lt lP,flag)
 dist=total(abs(P - lP)+abs(P-uP))

RETURN
END

;-----------------------------------------------
;   the following functions evaluate likelihood
;   values called from the minimization techniques
;   A function is provided for each set of interated
;   variables. Extra functions are necessary if the
;   line distances in multiplets are kept constant.
;-----------------------------------------------

FUNCTION FUNC_lik, P
; P(0..nlines-1)                llambda(0..nlines-1)
; P(nlines..2*nlines-1)         lsigma(0..nlines-1)

COMMON flag,cflag
if n_elements(cflag) eq 0 then cflag=0

@fit.inc

; insert penalties
  penalty,P,[lamstart,lsig],[lamend,usig],flag,dist
  if flag gt 0 then begin
     cflag=cflag+flag
     RETURN,1000.*dist*cflag
  endif

  callik,P(0:nlines-1),P(nlines:2*nlines-1),ampout,like,yc
  lik = like - likezero

 RETURN,lik
END

;-----------------------------------------------

FUNCTION FUNC_clik, P
; P(0)			llambda(0)
; P(1..nlines)		lsigma(0..nlines-1)

COMMON flag,cflag
if n_elements(cflag) eq 0 then cflag=0

@fit.inc

; insert penalties
  penalty,P,[lamstart(0),lsig],[lamend(nlines-1),usig],flag,dist
 if flag gt 0 then begin
     cflag=cflag+flag
     RETURN,1000.*dist*cflag
  endif 

;   restore constant line distance
llambda(0)=P(0)
FOR lline=1,nlines-1 DO llambda(lline)=llambda(lline-1)+steplam(lline-1)
  callik,llambda,P(1:nlines),ampout,like,yc
  lik = like - likezero

 RETURN,lik
END

;-----------------------------------------------

FUNCTION FUNC_lamlik, P
; P(0..nlines-1)                llambda(0..nlines-1)

COMMON flag,cflag
if n_elements(cflag) eq 0 then cflag=0

@fit.inc

; insert penalties  
  penalty,P,lamstart,lamend,flag,dist
 if flag gt 0 then begin
     cflag=cflag+flag
     RETURN,1000.*dist*cflag
  endif

  callik,P(0:nlines-1),lsigma,ampout,like,yc
  lik = like - likezero

 RETURN,lik
END

;-----------------------------------------------

FUNCTION FUNC_clamlik, P
; P                llambda(0)

COMMON flag,cflag
if n_elements(cflag) eq 0 then cflag=0

@fit.inc

; insert penalties
  penalty,P,lamstart(0),lamend(nlines-1),flag,dist
  if flag gt 0 then begin
     cflag=cflag+flag
     RETURN,1000.*dist*cflag
  endif

;   restore constant line distance
llambda(0)=P
FOR lline=1,nlines-1 DO llambda(lline)=llambda(lline-1)+steplam(lline-1)
  callik,llambda,lsigma,ampout,like,yc
  lik = like - likezero

 RETURN,lik
END

;-----------------------------------------------


FUNCTION FUNC_siglik, P
; P(0..nlines-1)	lsigma(0..nlines-1)

COMMON flag,cflag
if n_elements(cflag) eq 0 then cflag=0

@fit.inc

; insert penalties  
  penalty,P,lsig,usig,flag,dist
  if flag gt 0 then begin
     cflag=cflag+flag
     RETURN,1000.*dist*cflag
  endif

  nlines=n_elements(yc(*,0))
  callik,llambda,P(0:nlines-1),ampout,like,yc
  lik = like - likezero

 RETURN,lik
END

;-----------------------------------------------

FUNCTION FUNC_testlik, P
; this function is defined to give the chance of calculating
; likelihood values without changing the fit results globally
; (e.g. error analysis para_err)
; P(0..nlines-1)                llambda(0..nlines-1)
; P(nlines..2*nlines-1)         lsigma(0..nlines-1)

COMMON flag,cflag
if n_elements(cflag) eq 0 then cflag=0

@fit.inc

; insert penalties  
  penalty,P,[lamstart,lsig],[lamend,usig],flag,dist
  if flag gt 0 then begin
     cflag=cflag+flag
     RETURN,1000.*dist*cflag
  endif

  amp=ampout
  yctest=yc

  callik,P(0:nlines-1),P(nlines:2*nlines-1),amp,like,yctest
  lik = like - likezero

 RETURN,lik
END

;-----------------------------------------------

FUNCTION errlik,x
;  called from 'para_err':
;  determine difference of likelihood compared to hlik 
;  The root finder 'zbrent' finds the value x
;  where likelihood=hlik

COMMON erranal,P,index,hlik
;  P     =[lam(0)..lam(nlines-1),sig(0)..sig(nlines-1)]
;	 vector with fit results, i.e. dLik/dP=0
;  index indicates for which variable and for which line
;        the error is to be estimated
;  x     is the iterated variable provided from 'zbrent'
;  hlik  = min(likelihood)+dL, dL depending on the
;	 desired accuray for the error (c.f. para_err).

;  since only one argument can be provided from
;  functions called from internal routines (zbrent)
;  other values are passed via COMMON block erranal
;
Perr=P
Perr(index)=x
lik0=func_testlik(Perr)-hlik

RETURN,lik0
END

;-----------------------------------------------

FUNCTION cut,zahl,n
;  functions rounds floats and puts the result
;  into a string with n digits
;  usage:   string=cut(float,n)
;
if n gt 16 then message,'no more than 16 digits can be handled with cut'
ndigits=long(alog10(abs(zahl))+.999)
vz=zahl/abs(zahl)
if n gt 1 then zround=1.*(float(zahl)*10.^(n-ndigits)+vz*5)/10.^(n-ndigits) else zround =zahl
neu_string=STRMID(STRTRIM(STRING(float(zround),format='(f16)'),2),-2,n)
FOR i=0,n_elements(zahl)-1 DO if zahl(i) eq 0.0 then neu_string(i)='0.0'
return,neu_string
end

;-----------------------------------------------

FUNCTION sign, a1,a2
;  function used by 'brent'
return,a2/abs(a2)*abs(a1)
end

;-----------------------------------------------

function sbgfunc,p

@fit.inc
 sbg=p

 bbb = bbb + sbg
 if n_elements(likezero) eq 0 then likezero = likelihood(n_elements(nlines))

 chitemp = func_lik([llambda,lsigma]) ;calculate ampout
 bbb = bbb - sbg

 chi=bbb+sbg
 for i=0,count-1 DO chi(i) = chi(i) + total(yc(*,i)*ampout(*))
 chi2=0
 chicount=0
 for i=0,count-1 DO if errcount(i) gt 0. then begin
     chi2 = chi2 + (chi(i)-ncount(i))^2/errcount(i)^2
     chicount=chicount+1
   endif
 if chicount gt 0 then chi2=chi2/chicount else chi2=-1.

 return,chi2
end

;-----------------------------------------------

pro geter,xvec,yvec,delta,xmin,xerr,ymin,coeff
;   determine error from least-square polynomial fit
;   this routine can be called from 'para_err' alternatively
;   with 'zbrent' investigating the likelihood curve directly
;   'geter' is not recommended and my therefore be obsolete
;   (see also comments in 'para_err')

  x0=xvec(0)
  x1=xvec(1)
  x2=xvec(2)
  y0=yvec(0)
  y1=yvec(1)
  y2=yvec(2)

  p1=float(x2 * (y1-y0)+x1 * (y0-y2)+x0 * (y2-y1))
  p2=float(x2^2*(y1-y0)+x1^2*(y0-y2)+x0^2*(y2-y1))
  p3=float(x2^2*(x0-x1)+x1^2*(x2-x0)+x0^2*(x1-x2))

  coeff=fltarr(3)
  coeff(2)=p1/p3
  coeff(1)=-p2/p3
  coeff(0)=(x2^2*(x0*y1-x1*y0)+x1^2*(x2*y0-x0*y2)+x0^2*(x1*y2-x2*y1))/p3
;coeff = poly_fit(xvec,yvec,2)
;  polynom has form:
;       p  (x) = coeff(2)*x^2+coeff(1)*x+coeff(0)
;       p (x) = 2*coeff(2)*x+coeff(1)
;       p(x) = 2*coeff(2)
xmin = -0.5*coeff(1)/coeff(2)
ymin = poly(xmin,coeff)
xerr = sqrt(float(delta)/abs(coeff(2)))

return
end

;-----------------------------------------------

pro Amperr, yc,ampout,dellik,like0,like1,like2,eramp,Aerr
;
;  determine error in amplitude 'eramp'
;  for which second derivative 'like2' is used
;  first derivative 'like1' is not used but given
;  dellik = 1 for accuracy of error within 68.3%
;  (actually dellik=1/2, but this turns to 1 because
;   -2log(lik) is used.)
;
 COMMON DATintv,x,dx,ncount,backarr,errcount,count

 nlines=n_elements(yc(*,0))
 ndim=n_elements(yc(0,*))
 source=fltarr(ndim)
 eramp=fltarr(nlines)

FOR i=0,ndim-1 DO source(i)=backarr(i)+total(ampout*yc(*,i))
;;like0 = -2.*total(-source + float(ncount)*alog(source))

;FOR lline=0,nlines-1 DO BEGIN
; ycvec=reform(yc(lline,*))
;; like1 = -2.*(-1.+total(float(ncount)*ycvec/source))
; like2 = 2.*total(float(ncount)*ycvec*ycvec/(source*source))
; eramp(lline) = sqrt(2.*dellik/like2)
;ENDFOR

;print,'eramp=',eramp

Aerr=fltarr(nlines,nlines)
testvec=fltarr(nlines)
FOR lline=0,nlines-1 DO BEGIN
    ycvec=reform(yc(lline,*))
    for i=0,nlines-1 DO testvec(i)=double(total(double(ncount)*ycvec*reform(yc(i,*))/(source*source)))
    Aerr(lline,*)=testvec
ENDFOR

EV=fltarr(nlines,nlines)
eval=fltarr(nlines)
if nlines eq 1 then begin
   eval(0)=Aerr(0,0)
   EV(0)=1.
endif else eval=eigenql(Aerr,eigenvectors=EV)

EV=transpose(EV)
for lline =0,nlines-1 DO eramp(lline)=sqrt(dellik*total(dellik*EV(*,lline)*EV(*,lline)/eval(*)))

;print,'eramp=',eramp

return
end

;-----------------------------------------------

pro para_err,vec,vec_1,flag,dvec,delta,flag_err
;   determine error flag_err in vec with numerical method
;   flag     : =0 error for lambda, =1 for sigma
;   delta=1. : returns error with 68.3% precision
;   dvec     : shift in vec to get two other points on parabola
;    find point where likelihood value reaches minimum value + delta:
;      lik1=llik-delta			lik0=llik+delta
;	         \			  /
;		   \			/
;		       --   llik  ---
;
;      testm  	        vec(lline)         testp
; this is done numerically with zbrent procedure
;  the alternative (least-square polynomial fit 'geter') is less accurate
;  because the fitted parabolas are very steep such that small deviations
;  in x lead to large errors in y. The error value found thus depends on the
;  selected border values dvec+-delta.

;  flag indicates whether errors for sigma or lambda are to be found
;   I must admit this is a failed attempt to keep this routine flexible
;

COMMON fit_PARA,lamstart,lamend,nlines,prof,idlam,app_s,app_b,lsig,usig,sigunit,ilamit,isigit,sbgit,bgsmooth,dlam,beta

 COMMON out_PARA,imeth,linID,ires,iplot,ilab,ierrplot,ilog,nbin,nlik,ispline,iformat,iportrait
COMMON erranal,P,index,hlik

chkerr=0    ; =1 => perform check of error
ierrmeth=0  ; =1 => apply least square polonomial fit, else
;		 find error directly from data numerically with root finder
;	    ; =0 advised for steep likelihood curves

nlines=n_elements(vec)
flag_err=fltarr(nlines)
P=fltarr(2*nlines)

if flag ne 0 and flag ne 1 then message,'flag has to be one or zero in para_err'
if flag eq 0 then begin
   lamvec=vec
   sigvec=vec_1
   lvec=lamstart
   uvec=lamend
endif else begin
   lamvec=vec_1
   sigvec=vec
   lvec=lsig
   uvec=usig
endelse

;  point (vec,llik) for which error is to be determined
P=[lamvec,sigvec]
Ptest=P
llik=func_testlik(P)
hlik=llik+delta

;  find lower and upper points (vec_min,lik0) and (vec_plu,lik1)
 vec_min=vec-dvec
Ptest(flag*nlines)=vec_min
 lik0=func_testlik(Ptest)
 vec_plu=vec+dvec
Ptest(flag*nlines)=vec_plu
 lik1=func_testlik(Ptest)

;  determine error

FOR lline=0,nlines-1 DO BEGIN

getlik: if vec_min(lline) gt lvec(lline) and vec_plu(lline) lt uvec(lline) then begin
;    find point where likelihood value reaches minimum value+delta:
  if ierrmeth eq 0 then begin
   index=flag*nlines+lline  ; index of changed variable
   testm=zbrent(vec_min(lline),vec(lline),FUNC_NAME="errlik",TOLERANCE=1e-5)
   testp=zbrent(vec(lline),vec_plu(lline),FUNC_NAME="errlik",TOLERANCE=1e-5)
   test=max([vec(lline)-testm,testp-vec(lline)])
  endif else begin
   geter,[vec_min(lline)-vec(lline),0.,vec_plu(lline)-vec(lline)],[lik0,llik,lik1],delta,sigtest,test,llik1,coef
   if coef(2) le 0.0 then begin ;prevent negative parabolas by
	dvec(lline) = dvec(lline)/2.		;reducing delta
	if dvec(lline) gt .01 then begin
            vec_min=vec-dvec
            Ptest(flag*nlines)=vec_min
            lik0=func_testlik(Ptest)
            vec_plu=vec+dvec
            Ptest(flag*nlines)=vec_plu
            lik1=func_testlik(Ptest)
	    goto,getlik
	endif else test=0.
   endif
  endelse
  flag_err(lline)=test
;       check value of error
   if chkerr eq 1 then begin
     Ptest=P
     Ptest(flag*nlines+lline)=vec(lline)-flag_err(lline)
    lik0=func_testlik(Ptest)
    print,'s----',vec(lline)-flag_err(lline),' => ',cut(lik0,9)
    Ptest=P
    lik1=func_testlik([lamvec,sigvec])
    print,'s====',vec(lline),' => ',cut(lik1,9),' delta = ',cut(lik0-lik1,7)
     Ptest(flag*nlines+lline)=vec(lline)+flag_err(lline)
    lik2=func_testlik(Ptest)
    print,'s++++',vec(lline)+flag_err(lline),' => ',cut(lik2,9),' delta = ',cut(lik2-lik1,7)
   endif
 endif else begin
         dvec(lline) = dvec(lline)/2.
	 if dvec(lline) gt .01 then begin
            vec_min=vec-dvec
	    Ptest(flag*nlines)=vec_min
	    lik0=func_testlik(Ptest)
	    vec_plu=vec+dvec
	    Ptest(flag*nlines)=vec_plu
	    lik1=func_testlik(Ptest)
	    goto,getlik
	 endif else flag_err(lline)=0.
 endelse
ENDFOR

return
end

;-----------------------------------------------

pro brent,itmax,ax,bx,cx,tol,xmin,fw,fx,fv,idlam,iter,func
;
;	NumRec Routine to determine extremal value by parabolic fit
;	and golden section
;	  PARAMETERS:
;		itmax		maximum number of allowed iteration steps
;		ax,bx,cx	three x-values
;		fw,fx,fv	y-values determined in the program
;		tol		precision of the result
;		xmin		result
;		iter		number of used steps
;		func		character to choose between two functions
;
;   routine adapted from: Numerical Recipes in Fortran,
;			  second Edition 1992, p. 397-398
COMMON result,llambda,erlam,lsigma,ersig,ampout,eramp
;  nlines=n_elements(bx_vec)
;  test=fltarr(nlines)
;  test1=fltarr(nlines)
;  bx=bx_vec(lline)

  cgold=.3819660
  zeps=1.E-10

  a=min([ax,cx])
  b=max([ax,cx])
  v=bx
  w=v
  x=v
  e=0.
;  if func eq 'lam' then P=[bx_vec,lsigma]
;  if func eq 'sig' then P=[llambda,bx_vec]
  fx=call_function(func,bx)
  fv=fx
  fw=fx
  FOR iter=0,itmax-1 DO BEGIN
	xm=0.5*(a+b)
	tol1=tol*abs(x)+zeps
	tol2=2.*tol1
	if abs(x-xm) le tol2-.5*(b-a) then goto,enditer
	if abs(e) gt tol1 then BEGIN
	   r=(x-w)*(fx-fv)
	   q=(x-v)*(fx-fw)
	   p=(x-v)*q-(x-w)*r
	   q=2*(q-r)
;	   if q gt 0. then p=-p
	   p=-p*((q gt 0)*2-1)
	   q=abs(q)
	   etemp=e
	   e=d
	   if abs(p) ge abs(.5*q*etemp) or p le q*(a-x) or p ge q*(b-x) then goto,goldsec
	   d=p/q
	   u=x+d
	   if u-a lt tol2 or b-u lt tol2 then d=sign(tol1,xm-x)
	   goto,parafit
	endif
goldsec: e = a*(x ge xm) + b*(x lt xm) - x  ; optimized - direct way in comments
;	if x ge xm then BEGIN
;	   e=a-x
;	endif else begin
;	   e=b-x
;	endelse
	d=cgold*e
parafit:if abs(d) ge tol1 then begin
	   u=x+d
        endif else begin
           u=x+sign(tol1,d)
        endelse
        dx = u - bx
        bx = u
	fu=call_function(func,bx)

	if fu le fx then begin
	   if u ge x then begin
		a=x
	   endif else begin
		b=x
	   endelse
	   v=w
	   fv=fw
	   w=x
	   fw=fx
	   x=u
           bx= u
	   fx=fu
	endif else begin
	   if u lt x then begin
		a=u
	   endif else begin
		b=u
	   endelse
	   if fu le fw or w eq x then begin
		 v=w
		 fv=fw
		 w=u
		 fw=fu
	   endif else begin
		 if fu le fv or v eq x or v eq w then begin
		    v=u
		    fv=fu
		 endif
	   endelse
	endelse
  ENDFOR
  print,'brent exceeded maximum iterations of n=',itmax,' with ',func,'=',x
enditer: xmin=x
  brent=fx
  ax=w
  cx=v
  bx=x 
return
end

;-------------------------------------------------------

pro sbgiterate,sarr,sbgmed,sbg,sbgchi
;  determine source background by median (lines excluded)

COMMON medsig,medsig

nstep=20
;sbgmin=sbgmed-medsig*sqrt(sbgmed)

test=sarr
sbg=median(test)
i=0
 lsel=n_elements(sarr)
 sbgmax=sbg+medsig*sqrt(sbg)
 sel=where(test lt sbgmax,csel)
 if csel gt 0 then test=test(sel) 
 sbg=median(test)
while csel lt lsel and i lt nstep do begin
 lsel=csel
 i=i+1
 sbgmax=sbg+medsig*sqrt(sbg)
 sel=where(test lt sbgmax,csel)
 if csel gt 0 then test=test(sel)
 sbg=median(test)
endwhile

if sbgchi eq 1 then begin
  fw=sbgfunc(0.)
  fx=sbgfunc(sbgmed)
  fv=sbgfunc(sbgmax)
 brent,800,0.,sbgmed,sbgmax,1e-4,sbgmin,fw,fx,fv,idlam,iter,'sbgfunc'
 sbg=sbgmin
endif

return
end

;-----------------------------------------------

pro binit,vec1,vec2,vec3,vec4,vec1_bin,vec2_bin,vec3_bin,vec4_bin,nbin
;   four vectors are binned simultanously with same binning value nbin
;    the integral of the first vector will not be conserved!
;	(because it is treated as x-Axis)
nextr=nbin*long(1.*n_elements(vec1)/nbin)
ww1 = reform(vec1)
ww1 = extrac(ww1,0,nextr)
nextr=nbin*long(1.*n_elements(vec2)/nbin)
ww2 = reform(vec2)
ww2 = extrac(ww2,0,nextr)
nextr=nbin*long(1.*n_elements(vec3)/nbin)
ww3 = reform(vec3)
ww3 = extrac(ww3,0,nextr)
nextr=nbin*long(1.*n_elements(vec4)/nbin)
ww4 = reform(vec4)
ww4 = extrac(ww4,0,nextr)

num = n_elements(ww1)
vec1_bin=rebin(ww1,num/nbin)
num = n_elements(ww2)
vec2_bin=float(nbin)*rebin(ww2,num/nbin)
num = n_elements(ww3)
vec3_bin=float(nbin)*rebin(ww3,num/nbin)
num = n_elements(ww4)
vec4_bin=float(nbin)*rebin(ww4,num/nbin)

return
end

;-------------------------------------------------------

pro plotfit,fname,dtime,fname3,LOG=log
;	produce a nice plot with all parameters at the right place

@fit.inc

;         data will be binned with nbin
;       wav_s : binned wavelength vector
;       sp_s  : binned source count vector [cts]
;       sp_err: binned vector with errors [cts]
;       sp_b  : binned background count vector [cts]
  if nbin gt 1 then begin
     binit,wave(good),sp(good),err(good),bkgd(good),wav_s,sp_s,sp_err,sp_b,nbin
     nextr=nbin*long(1.*n_elements(bg0(good))/nbin)
     ww1 = reform(bg0(good))
     ww1 = extrac(ww1,0,nextr)
     num = n_elements(ww1)
     bg0_bin=float(nbin)*rebin(ww1,num/nbin)
  endif else begin
     wav_s=wave(good)
     sp_s=sp(good)
     sp_err=err(good)
     sp_b=bkgd(good)
     bg0_bin=bg0(good)
  endelse

  nlines=n_elements(yc(*,0))
  vsigma = fltarr(nlines) ; sigma in untis km/s
  ervsig = fltarr(nlines) ; ersig in untis km/s

;  determine plotrange
;  dlam=(max(lamend)-min(lamstart))/2.
  xmin=max([min(llambda)-dlam,min(wave(good))])
  xmax=min([max(llambda)+dlam,max(wave(good))])
  dlam=(xmax-xmin)/2.
  goodplot=where(wave ge xmin and wave le xmax,pcount)
  ndim=pcount
  if pcount eq 0 then begin
     print,'empty plot range'
     return
  endif
    goodbin =where(wav_s gt xmin and wav_s lt xmax)
    ycplot = fltarr(nlines,pcount)    
    xplot  = wave(goodplot)
    ncount_plot = sp(goodplot)
    bbb_plot  = bkgd(goodplot)
; rearrange yc corresponding to new wavelength range
  slplot=fltarr(nlines,pcount)
  yplot = 0.0
  if spec_only eq 0 then begin
   sel0=where(wave(good) eq wave(goodplot(0)))
   FOR lline=0,nlines-1 DO BEGIN
       FOR i=0,pcount-1 DO ycplot(lline,i) = yc(lline,i+sel0)
   ENDFOR
   FOR lline=0,nlines-1 DO BEGIN
	slplot(lline,*) = reform(ycplot(lline,*))*ampout(lline)+bbb_plot
        yplot = yplot + reform(ycplot(lline,*))*ampout(lline)
   ENDFOR
;reform(ycplot(lline,*))*ampout(lline)
   yplot = yplot + bbb_plot
; calculate chi^2 if error plotting enabled
   if ierrplot gt 0 then begin
      errpos=good(where(err(good) ne 0))
      chi2=total((yplot(errpos)-ncount_plot(errpos))^2/err(errpos)^2)/$
              n_elements(errpos)
   endif
  endif
if iplot eq 0 then return
   print, 'Do plotting...'

   wavetitle='!7k!3 / '+string("305B)
   wavetitle='!7k!3 ('+string("305B)+')'

;  multiply fitted curve yplot with nbin in order to
;	  draw the curve through data sp_s, which is binned with nbin.
;         The integral of yplot is not conserved now!
;	  (but integral of sp_s is correct)
   yplot=float(nbin)*yplot
   slplot=float(nbin)*slplot
   spmin=min([sp_s(goodbin),yplot])
   spmax=1.01*max([sp_s(goodbin),yplot])
   dummy=fix(abs(alog10(binsize*nbin)))+3
   dummy=5
   cpb='Counts/'+cut(binsize*nbin,dummy)+string("305B)+' bin'
   cpb='Counts/'+cut(1000.*binsize*nbin,dummy)+'m'+string("305B)+' bin'
   if exp ne 1. then cpb=cpb+'/'+cut(exp/1000.,5)+'ksec'
;spmax=spmax*1.2  ;can enhance yrange with left+right labels
   if KEYWORD_SET(LOG) THEN plot_io,wav_s,sp_s,xtitle=Wavetitle,$
		        ytitle=cpb,xthick=boxthick,ythick=boxthick,$
			charthick=boxthick,$
			xrange=[xmin,xmax],$
			yrange=[.9*spmin,1.1*max([max(yplot),spmax])],$
			title=linID,xstyle=1,charsize=cs,ys=1, $
			psym=10,linestyle=iline,thick=ithick $
		    else $
		      plot,wav_s,sp_s,xtitle=Wavetitle,ytitle=cpb,$
			xrange=[xmin,xmax],xthick=boxthick,ythick=boxthick,$
			charthick=boxthick,$
			yrange=[0,1.1*max([max(yplot),spmax])],$
			title=linID,xs=1,charsize=cs,ys=1, $
			psym=10,linestyle=iline,thick=ithick

    if spec_only eq 0 then begin
       oplot,xplot,yplot,linestyle=fline,thick=fthick
       oplot,wav_s,sp_b,linestyle=bline,thick=bthick
;writefile,wave(goodplot),sp(goodplot),yplot/nbin,bg0(goodplot),sqrt(bg0(goodplot)),'plot.dat'
       if aline ge 0 then for i=0,nlines-1 DO oplot,xplot,slplot(i,*),linestyle=aline,thick=athick
    endif
    if ierrplot gt 0 then oploterror,wav_s,sp_s,sp_err,psym=3,/nohat

; ++++++++++++++++  set text to be inserted in graphic +++++++++++++++
IF ilab eq 1 then begin
   index=sort(llambda)
   FOR l=0,nlines-1 DO begin
    lline=index(l)
; ------------ AMPLITUDE --------------
    if eramp(lline) gt 0. then begin 
       text1='A='+cut(ampout(lline),4)+' '+string("261B)+' '+cut(eramp(lline),3)
    endif else begin
      text1='A='+cut(ampout(lline),4)
    endelse
    xAtext=llambda(lline)
    test=n_elements(wave)-1
    test=wave(1:test)
    ml=max(test-wave) ; worst resolution
    yAtext=1.01*max(slplot(lline,where(xplot gt llambda(lline)-ml and xplot lt llambda(lline)+ml)))
    xyouts,xAtext,yAtext,text1,charsize=lcs,charthick=lth
;--------------- SIGMA ---------------
    vsigma(lline) = 300000.0*lsigma(lline)/llambda(lline)
    ervsig(lline) = 300000.0*ersig(lline)/llambda(lline)
    if ersig(lline) gt 1e-4 then begin
       text2='!7r!3 = '+cut(1000.*lsigma(lline),4)+' '+string("261B)+' '+cut(1000.*ersig(lline),4)+' m'+STRING("305B)
       text2a='!7r!3 = '+cut(vsigma(lline),6)+' '+string("261B)+' '+cut(ervsig(lline),6)+' km/s'
    endif else begin
       text2='!7r!3 = '+cut(1000.*lsigma(lline),4)+' m'+STRING("305B)
       text2a='!7r!3 = '+cut(vsigma(lline),6)+' '+' km/s'
    endelse
    xsigtext=xAtext+1.3*lsigma(lline)
    IF KEYWORD_SET(LOG) THEN ysigtext=min(yplot)+yAtext/5. else $
			     ysigtext=(yAtext+min(yplot))/2.
    yvsigtext=ysigtext-max(yplot)*.03
    if sigunit eq 0 then xyouts,xsigtext,ysigtext,text2,charsize=lcs,charthick=lth $
       else xyouts,xsigtext,yvsigtext,text2a,charsize=lcs,charthick=lth
;-------------- LAMBDA -----------------
;    textarrow='!95!3'

    xlamtext=llambda(lline)-0.85*(xmax-xmin)*binsize
    if lline gt 0 then if xlamtext lt oldxpos+0.85*binsize*(ldigs+edigs) and yflip eq 0. then yflip=.7 else yflip=0.
    if lline gt 1 then if xlamtext lt ooldxpos+0.85*binsize*(ldigs+oldedigs) then yflip=1.5
    if n_elements(oldxpos) ne 0 then ooldxpos=oldxpos
    oldxpos=xlamtext
    if n_elements(edigs) ne 0 then oldedigs=edigs

    ldigs=fix(alog10(llambda(lline)))+3
    edigs=fix(abs(alog10(erlam(lline))))+1

    if erlam(lline) gt 1e-5 then begin
       text3=cut(llambda(lline),7)+' '+string("261B)+' '+cut(erlam(lline),6)+' '+STRING("305B)
    endif else begin
       text3=cut(llambda(lline),7)+' '+STRING("305B)
    endelse
    xlamtext=llambda(lline)-0.085*(xmax-xmin)
    if 1./3.*l eq fix(l/3) then yflip=0. else if 1./3.*(l+1) eq fix((l+1)/3) then yflip = 2. else yflip = 1.
    IF KEYWORD_SET(LOG) THEN $
      ylamtext=alog10(.9*spmin+(spmax-spmin)*.01 + alog10(spmin)/100.0 * (spmax-spmin) * yflip) $
    else $
      ylamtext=(spmax-spmin) * 0.05 * yflip+.01*(spmax-spmin)
    xyouts,xlamtext,ylamtext,text3,charsize=lcs,charthick=lth

;  +++++++++++++++++++++++++++++++++++++++++++++++++++++

ENDFOR
; ----------- other interesting stuff -----------------

; right corner...
    ttext='t='+cut(dtime,4)+' sec'
;    numsigtext='num_sig='+cut(num_sig,3)
;    liktext='-2log!12L!3 = '+STRMID(STRTRIM(STRING(FORMAT='(F10.1)',llik),2),-2,10)
    sbgA=sbg/(wave(good(pcount-1))-wave(good(0)))*(pcount-1)
    sbgtext='BG!Isource!N='+cut(sbgA,4)+'cts/'+string("305B)
    if ierrplot gt 0 then chitext='!7v!3!U2!N/N='+cut(chi2,5)
  if fname3 ne 'NULL' then if n_elements(legtext) eq 0 then legtext=[fname3]
  if dtime gt 0. then if n_elements(legtext) eq 0 then legtext=[ttext] else legtext=[legtext,ttext]
  if sbg gt 0. then if n_elements(legtext) eq 0 then legtext=[sbgtext] else legtext=[legtext,sbgtext]
  if ierrplot gt 0 then if n_elements(legtext) eq 0 then legtext=[chitext] else legtext=[legtext,chitext]
;  legend,legtext,charsize=lcs,charthick=lth,/right
;left corner...
  legtext=['Iterated variables:']
  if ilamit eq 1 then if idlam gt 0 then legtext=[legtext,'!7k!3!Irestr.!N'] else legtext=[legtext,'!7k!3']
  if isigit eq 1 then legtext=[legtext,'!7r!3']
  if sbgit eq 1 then if sbgchi eq 1 then legtext=[legtext,'BG!ISource!N with !7v!3!U2!N'] else legtext=[legtext,'BG!ISource!N with median']
;  legend,legtext,charsize=lcs,charthick=lth
ENDIF

; restore integral
yplot=1.*yplot/float(nbin)

return
end

;-----------------------------------------------

pro input,unit,fname3

;  read parameter file and store into COMMON blocks

@fit.inc

if n_elements(wave) le 0 then read_data,unit,datafile

linID='?' ; pure initialization...

close,unit
openr,unit,fname3
readf,unit,vers
if vers ne version then message,'wrong version!'
readf,unit,linID
if strtrim(linID) eq 'NULL' then linID=''
readf,unit,nlines
nlines=fix(nlines)

llambda=fltarr(nlines)
lsigma=fltarr(nlines)
f=fltarr(nlines)
prof=intarr(nlines)
beta=fltarr(nlines)

;  skip negative values
readfirst: readf,unit,testl,tests,testf,testp,testfl,testfs,testll,testb
  if testl gt 0. then begin
     llambda(0) = testl
     lsigma(0)=tests
     f(0)=testf
     prof(0)=testp
     beta(0)=testb
  endif else goto,readfirst
	
lline=1
FOR i=1,nlines-1 DO begin
  readf,unit,testl,tests,testf,testp,testfl,testfs,testll,testb
  if testl gt 0. then begin
     llambda(lline) = testl
     lsigma(lline)=tests
     f(lline)=testf
     prof(lline)=testp
     beta(lline)=testb
     lline=lline+1
  endif else nlines=nlines-1
ENDFOR
if lline lt nlines then begin
     print,'nlines reduced to ',lline
     nlines=lline
endif
llambda=llambda(0:nlines-1)
lsigma=lsigma(0:nlines-1)

;  skip further negative values
readf,unit,testl,tests,testf
lpos: if testl lt 0. then begin
        readf,unit,testl,tests,testf
        goto,lpos
     endif
app_s = testl
app_b = tests
v=testf

medsig=3.
if vers gt 3.0 then readf,unit,sbgA,sbgit,bgsmooth,sbgchi,medsig else readf,unit,sbgA,sbgit,bgsmooth,sbgchi,medsig
sbg = sbgA*(wave(n_elements(wave)-1)-wave(0))/n_elements(wave)

readf,unit,dsig,sigunit,dlam
dlam=abs(dlam)
sigunit=fix(sigunit)
dsig=abs(dsig)
readf,unit,ilamit,idlam,iord,idsig
readf,unit,isigit
readf,unit,ires
readf,unit,iformat,iportrait
spec_only=0
if version gt 1.1 then readf,unit,iplot,ilab,ierrplot,ilog,nbin,spec_only $
else readf,unit,iplot,ilab,ierrplot,ilog,nbin
readf,unit,nlik,ispline
close,unit

ilamit=fix(ilamit)
isigit=fix(isigit)
sbgit=fix(sbgit)
ires=fix(ires)
iformat=fix(iformat)
iportrait=fix(iportrait)
ilog=fix(ilog)
ilab=fix(ilab)
ierrplot=fix(ierrplot)
iplot=fix(iplot)
nbin=fix(nbin)
spec_only=fix(spec_only)
nlik=fix(nlik)
ispline=fix(ispline)
idlam=fix(idlam)

;  prevent fitting if only plotting of spectrum is desired 
if spec_only eq 1 then begin
	sbgit=0
	bgsmooth=0
	ilamit=0
	isigit=0
	iplot=1
	ilab=0
endif

;  Selecting method is obsolete now!
imeth=2

;  set start values
if sigunit eq 1 then begin
   dsig   = llambda(0)*dsig/300000.       ; dsig, given in km/s --> Angstroem
   for lline=0,nlines-1 DO lsigma(lline) = llambda(lline)*lsigma(lline)/300000.
endif
if max(llambda*iord) gt max(wave) then begin
	print,'WARNING:'
	print,'=======  ',fix(iord),' order =',max(llambda*iord),STRING("305B),' is outside spectrum'
	print,'will investigate first order'
endif else if iord gt 0. then llambda=llambda*iord

;FOR lline=0,nlines-1 DO if lsigma(lline) lt 0.0 then lsigma(lline)=lsigma(0)
lsigma = lsigma*(lsigma gt 0.0) +lsigma(0)*(lsigma le 0.0)  ; the idl-shortcut

;   prevent negative values of sigma
if dsig gt min(lsigma) then dsig=min(lsigma)

; define borders of sigma/lambda
lsig=lsigma-dsig
usig=lsigma+dsig
lamstart = llambda - dlam
lamend   = llambda + dlam

;  consistency checks
if nlines eq 1 or ilamit eq 0 then idlam=0

if idlam eq 1 and ilamit gt 0 and isigit eq 0 then imeth=0
if isigit+ilamit eq 1 and nlines eq 1 then imeth=0

return
end

;-----------------------------------------------

pro lamran,count

;  determine datainterval 'good' to be investigated
 COMMON DATA,wave,sp,err,bkgd,good,yc,exp
 COMMON result,llambda,erlam,lsigma,ersig,ampout,eramp,llik,likezero
 COMMON fit_PARA,lamstart,lamend,nlines,prof,idlam,app_s,app_b,lsig,usig,sigunit,ilamit,isigit,sbgit,bgsmooth,dlam,beta


 maxshift=2.
 maxsigma=max(lsigma)+(usig(0)-lsig(0))/2.
 num_sig=3

 lam_l = min(lamstart)
 lam_o = max(lamend)

 diff_lam = lam_o-lam_l
 lambda=(lam_l+lam_o)/2.
 lam_l = lambda - maxshift - float(num_sig)*maxsigma
 lam_o = lambda + maxshift + float(num_sig)*maxsigma
 if diff_lam gt lam_o-lam_l then begin
	num_sig = diff_lam/2./maxsigma
	lam_l = min(lamstart)-maxshift
	lam_o = max(lamend)+maxshift
        print,'had to change num_sig to num_sig=',num_sig,' in order to catch all lines'
 endif

 good = where(wave ge lam_l and wave le lam_o,count)

return
end

;-----------------------------------------------

pro plotlik,P,ival,lline,vmin,vmax,likvec
; create a curve with likelihood values
; P=[llambda,lsigma]
; vmin,vmax:	step from vmin to vmax
; ival:		=0 step through lambda
;		=1 step through sigma
; lline:	select one line (0 for first line)
; likvec:	vector with likelihood values for output

COMMON BG,bg0,sbg
COMMON fit_PARA,lamstart,lamend,nlines,prof,idlam,app_s,app_b,lsig,usig,sigunit,ilamit,isigit,sbgit,bgsmooth,dlam,beta
COMMON out_PARA,imeth,linID,ires,iplot,ilab,ierrplot,ilog,nbin,spec_only,nlik,ispline,iformat,iportrait

xv=fltarr(nlik)
likvec=fltarr(nlik)
dv=(vmax-vmin)/nlik

if ival eq 0 then begin
   nv=lline
   xtext='!7k!3'+cut(lline,1)+' ['+STRING("305B)+']'
endif
if ival eq 1 then begin
   nv=nlines+lline
   xtext='!7r!3'+cut(lline,1)+' ['+STRING("305B)+']'
endif

FOR i=0,nlik-1 DO BEGIN
    xv(i)=vmin+i*dv
    P(nv)=xv(i)
    lik=func_testlik(P)
    likvec(i)=lik
ENDFOR

;  kick NaN-values out of vector
nsel=where(finite(likvec) eq 0,pcount)
if pcount gt 0 then likvec(nsel)=1

if ispline gt 0 then begin
   xvs=findgen((xv(nlik-1)-xv(0))*1000)/1000.+xv(0)
   vspline = SPLINE(xv,likvec,xvs)
   minlik=min(vspline,xmin)
   minval=xvs(xmin)
   plot,xvs,vspline,xtitle=xtext,ytitle='-2*log(lik)',title='Min at '+cut(minval,6),charsize=1.3
endif else begin
   minlik=min(likvec,xmin)
   minval=xv(xmin)
   plot,xv,likvec,xtitle=xtext,ytitle='-2*log(lik)',title='Min at '+cut(minval,6),charsize=1.3
endelse
P(nv)=minval

return
end

;----------------------------------

pro smoothbg,x_vec,y_vec,s_vec,m

;  smoothe vector y_vec with m-order polonomial

ndim=n_elements(y_vec)
s_vec=fltarr(ndim)

c=poly_fit(x_vec,y_vec,m)
s_vec=poly(x_vec,c)

return
end
